package fishrungames.bashgid.core.db;

import java.util.ArrayList;
import java.util.Collections;

import fishrungames.bashgid.MainActivity;

import fishrungames.bashgid.core.AlbumManager.AlbumFullData;
import fishrungames.bashgid.core.AlbumManager.AlbumRecordData;
import fishrungames.bashgid.core.AlbumManager.AlbumShortData;
import fishrungames.bashgid.core.ArticleManager;
import fishrungames.bashgid.core.ArticleManager.ArticleShortData;
import fishrungames.bashgid.core.JournalManager;
import fishrungames.bashgid.core.JournalManager.JournalRecordData;
import fishrungames.bashgid.core.JournalManager.JournalSemiFullData;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;

import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

public class JournalDataSource
{

	private BashgidSqliteHelper dbHelper = null;

	public JournalDataSource(Context context, BashgidSqliteHelper dbHelper)
	{
		this.dbHelper = dbHelper;

	}

	public void CreateNewJournal(JournalManager.JournalRecordData recordData)
	{
		SQLiteDatabase database = dbHelper.getWritableDatabase();

		if (database != null)
		{

			try
			{
				ContentValues values = new ContentValues();

				values.put(BashgidSqliteHelper.COLUMN_NAME, recordData.name);
				values.put(BashgidSqliteHelper.COLUMN_TITLE, recordData.title);
				values.put(BashgidSqliteHelper.COLUMN_DESCRIPTION, recordData.description);

				if (!innerIsJournalAlreadyExist(recordData.name, database))
				{
					database.insert(BashgidSqliteHelper.TABLE_JOURNAL, null, values);
				} else
				{
					// Replace
					database.update(BashgidSqliteHelper.TABLE_JOURNAL, values, BashgidSqliteHelper.COLUMN_NAME + " = " +"\"" + recordData.name +"\"", null);
				}

			} finally
			{
				dbHelper.close();
			}
		}

	}

	public boolean IsJournalAlreadyExist(String name)
	{
		boolean result = false;

		SQLiteDatabase database = dbHelper.getWritableDatabase();

		if (database != null)
		{

			try
			{
				result = innerIsJournalAlreadyExist(name, database);

			} finally
			{
				dbHelper.close();
			}
		}

		return result;

	}

	public void AddArticleToJournal(String articleName, String journalName)
	{
		SQLiteDatabase database = dbHelper.getWritableDatabase();

		if (database != null)
		{

			try
			{
				ContentValues values = new ContentValues();

				values.put(BashgidSqliteHelper.COLUMN_ARTICLE_NAME, articleName);
				values.put(BashgidSqliteHelper.COLUMN_JOURNAL_NAME, journalName);

				if (!InnerArticleJournalRelationAlreadyExist(articleName, journalName, database))
				{
					//Log.e("a", "aaa1");
					database.insert(BashgidSqliteHelper.TABLE_ARTICLE_JOURNAL_RELATION, null, values);
					//Log.e("a", "aaa2");
				}
				//Log.e("a", "aaa3");

			} finally
			{
				dbHelper.close();
			}
		}
	}


	public ArrayList<ArticleShortData> GetRecentArticles(String journalName, int count)
	{
		ArrayList<ArticleShortData> result = new ArrayList<ArticleShortData>();

		SQLiteDatabase database = dbHelper.getWritableDatabase();

		if (database != null)
		{
			try
			{

				JournalRecordData recordData = innerGetJournalByName(journalName, database);

				if (recordData != null)
				{

					ArrayList<String> articleNameArr = innerGetArticleArrInJournal(journalName, database);

					for (String articleName : articleNameArr)
					{
						result.add(MainActivity.getInstance().articleDataSource.InnerGetArticleShortData(articleName, database));
					}

					Collections.sort(result, new ArticleDataSource.DateSortComparator());

					if (result.size() > count)
					{
						result.subList(count, result.size() - count - 1).clear();
					}
				}

			} finally
			{
				dbHelper.close();
			}

		}

		return result;
	}
	
	public JournalSemiFullData GetJournalSemiFullData(String name)
	{
		return GetJournalSemiFullData(name, null, 0);
	}
	
	public JournalSemiFullData GetJournalSemiFullData(String name, int count)
	{
		return GetJournalSemiFullData(name, null, count);
	}

	public JournalSemiFullData GetJournalSemiFullData(String name, String startAfterArticleName, int count)
	{
		JournalSemiFullData result = null;

		SQLiteDatabase database = dbHelper.getWritableDatabase();

		if (database != null)
		{
			try
			{

				result = InnerGetJournalSemiFullData(name, database, startAfterArticleName, count);

			} finally
			{
				dbHelper.close();
			}
		}

		return result;
	}

	public ArrayList<JournalSemiFullData> GetAllJournalList()
	{
		ArrayList<JournalSemiFullData> result = new ArrayList<JournalSemiFullData>();
		SQLiteDatabase database = dbHelper.getWritableDatabase();

		if (database != null)
		{
			try
			{

				Cursor cursor = database.query(BashgidSqliteHelper.TABLE_JOURNAL, new String[] { BashgidSqliteHelper.COLUMN_NAME,
						BashgidSqliteHelper.COLUMN_TITLE, BashgidSqliteHelper.COLUMN_DESCRIPTION }, null, null, null, null, null, null);

				if (cursor != null)
				{
					if (cursor.moveToFirst())
					{

						do

						{

							String name = cursor.getString(0);
							if (!name.equals("news") && !name.equals("map"))
							{

								JournalSemiFullData journal = new JournalSemiFullData(new JournalRecordData(cursor.getString(0), cursor.getString(1), cursor.getString(2)));

								ArrayList<String> articleArr = innerGetArticleArrInJournal(cursor.getString(0), database);

								for (String article : articleArr)
								{
									journal.articleArr.add(MainActivity.getInstance().articleDataSource.InnerGetArticleShortData(article, database));

									/*
									String imageUrl = "R.drawable.no_picture";

									//Find first photo in first album (if exist)
									ArrayList<String> albumArr = MainActivity.getInstance().articleDataSource.innerGetAlbumNameArrInArticle(article, database);
									if (albumArr.size() > 0)
									{
										AlbumFullData album = MainActivity.getInstance().albumDataSource.InnerGetAlbumFullData(albumArr.get(0), database);

										if (album.photoRecordArr.size() > 0)
										{
											imageUrl = album.photoRecordArr.get(0).imageUrl;
										}
									}

									journal.previewImageArr.add(imageUrl);*/

								}


								result.add(journal);

							}


						} while (cursor.moveToNext());



					}
				}
			}
			finally
			{
				dbHelper.close();
			}
		}


		return result;
	}

	public JournalSemiFullData InnerGetJournalSemiFullData(String name, SQLiteDatabase database, String startAfterArticleName, int count)
	{

		JournalSemiFullData result = null;

		JournalRecordData recordData = innerGetJournalByName(name, database);

		if (recordData != null)
		{

			result = new JournalSemiFullData(recordData);

			ArrayList<String> articleArr = innerGetArticleArrInJournal(name, database);

			ArrayList<ArticleShortData> intermediateArticleArr = new ArrayList<ArticleShortData>();
			
			for (String article : articleArr)
			{
				
				intermediateArticleArr.add(MainActivity.getInstance().articleDataSource.InnerGetArticleShortData(article, database));
	
			}
			
			Collections.sort(intermediateArticleArr, new ArticleManager.ArticleShortDataTimeSortComparator());
			
			boolean articleCanGo = false;

			if (startAfterArticleName == null || startAfterArticleName.length() == 0)
			{
				articleCanGo = true;
			}

			int i = 0;
			
			for (ArticleShortData article : intermediateArticleArr)
			{
				if (!articleCanGo)
				{
					if (article.name.equals(startAfterArticleName))
					{
						articleCanGo = true;
					}
				}
				else
				{
					if (count == 0 || i < count)
					{
						result.articleArr.add(article);
						i++;
					}
						
				}
			}
		}

		return result;
	}


	private boolean innerIsJournalAlreadyExist(String name, SQLiteDatabase database)
	{
		boolean result = false;

		Cursor cursor = database.query(BashgidSqliteHelper.TABLE_JOURNAL, new String[] { BashgidSqliteHelper.COLUMN_NAME }, BashgidSqliteHelper.COLUMN_NAME
				+ "=?", new String[] { name }, null, null, null, null);

		if (cursor != null)
		{
			if (cursor.getCount() > 0)
			{
				result = true;
			}

			cursor.close();
		}

		return result;

	}

	public boolean InnerArticleJournalRelationAlreadyExist(String articleName, String journalName, SQLiteDatabase database)
	{
		boolean result = false;

		Cursor cursor = database.query(BashgidSqliteHelper.TABLE_ARTICLE_JOURNAL_RELATION, new String[] { BashgidSqliteHelper.COLUMN_ARTICLE_NAME,
				BashgidSqliteHelper.COLUMN_JOURNAL_NAME }, BashgidSqliteHelper.COLUMN_ARTICLE_NAME + "=?" + " AND " + BashgidSqliteHelper.COLUMN_JOURNAL_NAME + "=?",
				new String[] { articleName, journalName }, null, null, null, null);

		if (cursor != null)
		{
			if (cursor.getCount() > 0)
			{
				result = true;
			}

			cursor.close();
		}

		return result;

	}


	private ArrayList<String> innerGetArticleArrInJournal(String name, SQLiteDatabase database)
	{
		ArrayList<String> result = new ArrayList<String>();

		Cursor cursor = database.query(BashgidSqliteHelper.TABLE_ARTICLE_JOURNAL_RELATION, new String[] { BashgidSqliteHelper.COLUMN_ARTICLE_NAME },
				BashgidSqliteHelper.COLUMN_JOURNAL_NAME + "=?", new String[] { name }, null, null, null, null);

		if (cursor != null)
		{
			if (cursor.moveToFirst())
			{
				do
				{
					result.add(cursor.getString(0));
				} while (cursor.moveToNext());

			}

			cursor.close();
		}

		return result;

	}

	public JournalRecordData innerGetJournalByName(String name, SQLiteDatabase database)
	{

		JournalRecordData result = null;

		Cursor cursor = database.query(BashgidSqliteHelper.TABLE_JOURNAL, new String[] { BashgidSqliteHelper.COLUMN_NAME, BashgidSqliteHelper.COLUMN_TITLE,
				BashgidSqliteHelper.COLUMN_DESCRIPTION }, BashgidSqliteHelper.COLUMN_NAME + "=?", new String[] { name }, null, null, null, null);

		if (cursor != null)
		{
			if (cursor.getCount() > 0)
			{
				cursor.moveToFirst();

				result = new JournalRecordData(cursor.getString(0), cursor.getString(1), cursor.getString(2));
			}
			cursor.close();
		}

		return result;

	}

}
